;;;; This file is part of XMLTools
;;;; Copyright (C) 2004, 2007, 2015 Sergey Poznyakoff
;;;;
;;;; XMLTools is free software; you can redistribute it and/or modify
;;;; it under the terms of the GNU General Public License as published by
;;;; the Free Software Foundation; either version 3 of the License, or
;;;; (at your option) any later version.
;;;;
;;;; XMLTools is distributed in the hope that it will be useful,
;;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;;; GNU General Public License for more details.
;;;;
;;;; You should have received a copy of the GNU General Public License
;;;; along with XMLTools.  If not, see <http://www.gnu.org/licenses/>.
;;;;
(define-module (xmltools dict))

(define ->list)
(define ->string)

(catch 'misc-error
       (lambda ()
	 (use-modules (ice-9 iconv)
		      (rnrs bytevectors))
	 (set! ->list
	       (lambda (s)
		 (bytevector->u8-list (string->utf8 s))))
	 (set! ->string
	       (lambda (s)
		 (utf8->string (u8-list->bytevector s)))))
       (lambda (key . args)
	 (set! ->list string->list)
	 (set! ->string
	       (lambda (s)
		 (list->string (map integer->char s))))))
       
(define-public (dict:encode-string str)
    (apply
     string-append
     (map
      (lambda (x)
	(let ((n (if (char? x) (char->integer x) x)))
	  (if (or (and (> n 97) (< n 122))
		  (and (> n 65) (< n 90))
		  (and (> n 48) (< n 57)))
	      (string (if (char? x) x (integer->char x))) 
	      (let ((s (number->string n 16)))
		(string-append "%"
			       (if (< n 16)
				   "0" "")
			       s)))))
      (->list str))))

(define-public (dict:decode-string str)
  (do ((i 0)
       (sl '()))
      ((= i (string-length str)) (->string (reverse sl)))
    (let ((c (string-ref str i)))
      (set! sl
	    (cons
	     (cond
	      ((char=? c #\%)
	       (set! i (+ i 3))
	       (string->number (substring str (- i 2) i) 16))
	      (else
	       (set! i (1+ i))
	       (char->integer c)))
	     sl)))))
